/*
 *  IBM Confidential
 *  OCO Source Materials
 *
 *  (C) Copyright IBM Corp. 2005
 *
 *  The source code for this program is not published or otherwise divested
 *  of its trade secrets, irrespective of what has been deposited with the
 *  U.S. Copyright Office.
 *
 *
 */
package com.ibm.wtci.samples.axis.source;

import com.ibm.wtci.samples.axiswsn10.wsa.AttributedURI;
import com.ibm.wtci.samples.axiswsn10.wsa.EndpointReferenceType;
import com.ibm.wtci.samples.axiswsn10.wsn.NotificationConsumer;
import com.ibm.wtci.samples.axiswsn10.wsn.NotificationConsumerService;
import com.ibm.wtci.samples.axiswsn10.wsn.NotificationConsumerServiceLocator;
import com.ibm.wtci.samples.axiswsn10.wsn.NotificationMessageHolderType;
import com.ibm.wtci.samples.axiswsn10.wsn.TopicExpressionType;

import org.apache.axis.encoding.TypeMapping;
import org.apache.axis.encoding.TypeMappingRegistry;
import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.EventFactory;
import org.eclipse.hyades.logging.events.cbe.EventFactoryFactory;
import org.eclipse.hyades.logging.events.cbe.Situation;
import org.eclipse.hyades.logging.events.cbe.util.EventFormatter;

import java.net.URL;

import javax.xml.namespace.QName;

/**
 * Sample code for a WS-Notification 1.0 notification producer.
 * <p>
 * This sample is based on code generated using the Apache Axis 
 * wsdl2java tool.
 * 
 * @version $Header: /cvsroot/mainstream/wtci/code/samples/wsnt-axis/src/com/ibm/wtci/samples/axis/source/EventSourceWsn10.java,v 1.1 2005/12/08 19:15:38 nastacio Exp $ $Revision: 1.1 $
 * 
 * @since tec_3.9.0_fo1
 */
public class EventSourceWsn10 {

    /**
     * Main method for the event source.
     * 
     * @param args
     *            arguments passed from the command line
     */
    public static void main(String args[]) {
        EventSourceWsn10 source = new EventSourceWsn10();
        try {
            source.sendEvent();
            System.out.println("Notification sent");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Sends a notification message.
     * <p>
     * The request is compliant with the <code>NotificationConsumer</code>
     * port defined in the WS-Notification 1.0 specification.
     * 
     * @throws Exception
     *             if the notification cannot be sent for whatever reason.
     */
    private void sendEvent() throws Exception {
        //The first step is creating an event
        CommonBaseEvent[] events = createEvents();

        //Creating the Notification message encapsulating the event
        NotificationMessageHolderType[] notificationArray = 
            createNotificationMessage(events);

        // Obtaining the address of the NotificationConsumer10Service 
        // Web Services
        String endpoint = System.getProperty("service.address");
        if (endpoint == null) {
            //If no address was specified, the webservice is assumed to be
            // runnning in the localhost at port 8080 for axis
            endpoint = "http://localhost:8080/axis/services/NotificationConsumer10Soap";
        }

        //Creating an URL object for the address obtained
        URL serviceURL = new URL(endpoint);

        //Creating an instance of NotificationConsumerServiceLocator
        NotificationConsumerService notifierSvc = 
            new NotificationConsumerServiceLocator();

        //Using the NotificationConsumerService to obtain
        // NotificationConsumer object for the service
        NotificationConsumer notifier = 
            notifierSvc.getNotificationConsumer10Soap(serviceURL);

        //Registering an instance of CbeSerializer object for serializing
        // objects of
        //type CommonBaseEvent to be sent over the wire
        TypeMappingRegistry reg = (TypeMappingRegistry) 
            notifierSvc.getTypeMappingRegistry();
        TypeMapping tm = (TypeMapping) reg.getTypeMapping(endpoint);
        QName qn = new QName("http://www.ibm.com/AC/commonbaseevent1_0_1",
                "CommonBaseEvent");
        tm.register(CommonBaseEvent.class, qn, 
                    new CbeSerializerFactory(), null);

        //Sending the notification
        notifier.notify(notificationArray);
    }

    /**
     * Returns an array of CommonBaseEvent instances.
     * <p>
     * This method looks at the system property "cbe.file" and attempts to read
     * the events from the file indicated in the property. 
     * If the property is not set, an array with one hardcoded event is 
     * returned.
     * 
     * @return an array of event instances
     * 
     * @throws Exception
     *             if the events cannot be created for whatever reason
     */
    public static CommonBaseEvent[] createEvents() throws Exception {
        CommonBaseEvent[] events = null;

        String cbeFile = System.getProperty("cbe.file");
        if (cbeFile != null) {
            events = EventFormatter.eventsFromCanonicalXMLDoc(cbeFile);
        } else {
            // The first step is accessing the event factory
            EventFactory eventFactory = 
                EventFactoryFactory.createEventFactory();

            // Creating an event with an extension name.
            CommonBaseEvent event = 
                eventFactory.createCommonBaseEvent("EVENT");
            event.setCreationTimeAsLong(System.currentTimeMillis());

            // Setting the mandatory situation description for the event.
            Situation situation = eventFactory.createSituation();
            situation.setCategoryName(Situation.REPORT_SITUATION_CATEGORY);
            situation.setReportSituation("INTERNAL", "Succeeded");
            event.setSituation(situation);

            // Setting the mandatory component identification for the
            // event source
            event.setSourceComponentId("Event Source",
                "source.EventSource",
                "createEvent()",
                "http://www.ibm.com/namespaces/autonomic/Tivoli/Samples",
                "Sample",
                "unknown",
                "Hostname");

            // Setting Common Base Event version
            event.setVersion("1.0.1");
            // Setting optional fields
            event.setSeverity((short) 10);
            event.setMsg("Common Event Infrastructure Tutorial");
            event.validate();
            
            events = new CommonBaseEvent[] { event };
        }

        return events;
    }

    /**
     * Creates a WS-Notification message encapsulating a events of type
     * CommonBaseEvent.
     * 
     * @param events
     *            array of events to be carried inside the message.
     * 
     * @return an array of notification messages containg the events to be sent.
     * 
     * @throws Exception     if the notification cannot be created for
     *                       whatever reason. 
     */
    public static NotificationMessageHolderType[] createNotificationMessage(
            CommonBaseEvent events[]) throws Exception {
        NotificationMessageHolderType[] notificationArray = 
            new NotificationMessageHolderType[events.length];
        for (int i = 0; i < events.length; i++) {

            //Creating an instance of NotificationMessageHolderType
            notificationArray[i] = new NotificationMessageHolderType();

            //Creating a Topic element with name 'Topic' and value of
            //dialect attribute as 'tec'
            MessageElement[] msgTopic = new MessageElement[1];
            msgTopic[0] = new MessageElement();
            msgTopic[0].setName("Topic");
            msgTopic[0].setAttribute("dialect", "none");
            TopicExpressionType topicExpType = new TopicExpressionType();
            topicExpType.set_any(msgTopic);

            //Setting the Topic of the notification
            notificationArray[i].setTopic(topicExpType);

            //Setting the event to be the message of the notification
            notificationArray[i].setMessage(events[i]);

            //Setting information about the producer of the event in
            //the notification
            EndpointReferenceType endPointRef = new EndpointReferenceType();
            AttributedURI au = new AttributedURI();
            URI value = new URI("protocol", "your.event.source.address");
            au.set_value(value);
            endPointRef.setAddress(au);
            notificationArray[i].setProducerReference(endPointRef);
        }

        return notificationArray;
    }

}
