/*
 *  IBM Confidential
 *  OCO Source Materials
 *
 *  (C) Copyright IBM Corp. 2005
 *
 *  The source code for this program is not published or otherwise divested
 *  of its trade secrets, irrespective of what has been deposited with the
 *  U.S. Copyright Office.
 *
 *
 */
package com.ibm.wtci.samples.was.source;

import com.ibm.wtci.samples.waswsn10.emitter.NotificationConsumerService;
import com.ibm.wtci.samples.waswsn10.emitter.NotificationConsumerServiceLocator;
import com.ibm.wtci.samples.waswsn10.wsn.NotificationConsumer;
import com.ibm.wtci.samples.waswsn10.wsn.NotificationMessageHolderType;

import com.ibm.ws.webservices.engine.xmlsoap.SOAPElement;
import com.ibm.ws.webservices.engine.xmlsoap.SOAPFactory;

import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.EventFactory;
import org.eclipse.hyades.logging.events.cbe.EventFactoryFactory;
import org.eclipse.hyades.logging.events.cbe.Situation;
import org.eclipse.hyades.logging.events.cbe.util.EventFormatter;

import java.net.URL;

/**
 * Sample code for a WS-Notification 1.0 event producer.
 * <p>
 * This sample is based on code generated using the WebSphere wsdl2java tool.
 * 
 * @version $Header: /cvsroot/mainstream/wtci/code/samples/wsnt-was/src/com/ibm/wtci/samples/was/source/EventSourceWsn10.java,v 1.1 2005/12/08 19:23:26 nastacio Exp $ $Revision: 1.1 $
 * 
 * @since tec_3.9.0_fo1
 */
public class EventSourceWsn10 {

    /**
     * Main method for the event source.
     * 
     * @param args
     *            arguments passed from the command line
     */
    public static void main(String args[]) {
        EventSourceWsn10 source = new EventSourceWsn10();
        try {
            source.sendEvent();
            System.out.println("Notification sent");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Sends a notification message.
     * <p>
     * The request is compliant with the <code>NotificationConsumer</code>
     * port defined in the WS-Notification 1.0 specification.
     * 
     * @throws Exception
     *             if the notification cannot be sent for whatever reason.
     */
    private void sendEvent() throws Exception {
        //The first step is creating an event
        CommonBaseEvent[] events = createEvents();

        //Creating the Notification message encapsulating the event
        NotificationMessageHolderType[] notification = 
            createNotificationMessage(events);

        //Obtaining the address of the NotificationConsumerService webservice
        String endpoint = System.getProperty("service.address");
        if (endpoint == null) {
            // If no address was specified, the webservice is assumed to be
            // runnning in the localhost at port 9080 for was
            endpoint = "http://localhost:9080/wsnt-was/services/NotificationConsumer10Soap";
        }

        //Creating an URL object for the address obtained
        java.net.URL serviceURL = new URL(endpoint);

        //Creating an instance of NotificationConsumerServiceLocator
        NotificationConsumerService notifierSvc = 
            new NotificationConsumerServiceLocator();
        NotificationConsumer notifier = 
            notifierSvc.getNotificationConsumer10Soap(serviceURL);

        //Sending the notification
        notifier.notify(notification);

    }

    /**
     * Returns an array of CommonBaseEvent instances.
     * <p>
     * This method looks at the system property "cbe.file" and attempts to read
     * the events from the file indicated in the property. If the property is
     * not set, an array with one hardcoded event is returned.
     * 
     * @return an array of event instances
     * 
     * @throws Exception
     *             if the events cannot be created for whatever reason
     */
    public static CommonBaseEvent[] createEvents() throws Exception {
        CommonBaseEvent[] events = null;

        String cbeFile = System.getProperty("cbe.file");
        if (cbeFile != null) {
            events = EventFormatter.eventsFromCanonicalXMLDoc(cbeFile);
        } else {
            // The first step is accessing the event factory
            EventFactory eventFactory = EventFactoryFactory.createEventFactory();

            // Creating an event with an extension name.
            CommonBaseEvent event = eventFactory.createCommonBaseEvent("EVENT");
            event.setCreationTimeAsLong(System.currentTimeMillis());

            // Setting the mandatory situation description for the event.
            Situation situation = eventFactory.createSituation();
            situation.setCategoryName(Situation.REPORT_SITUATION_CATEGORY);
            situation.setReportSituation("INTERNAL", "Succeeded");
            event.setSituation(situation);

            // Setting the mandatory component identification for the
            // event source
            event.setSourceComponentId("Event Source",
                "source.EventSource",
                "createEvent()",
                "http://www.ibm.com/namespaces/autonomic/Tivoli/Samples",
                "Sample",
                "unknown",
                "Hostname");

            // Setting Common Base Event version
            event.setVersion("1.0.1");
            // Setting optional fields
            event.setSeverity((short) 10);
            event.setMsg("Common Event Infrastructure Tutorial");
            event.validate();

            events = new CommonBaseEvent[] { event };
        }

        return events;
    }

    /**
     * Creates a WS-Notification message encapsulating a events of type
     * CommonBaseEvent.
     * 
     * @param events
     *            array of events to be carried inside the message.
     * 
     * @return an array of notification messages containg the events to be sent.
     * 
     * @throws Exception     if the notification cannot be created for
     *                       whatever reason. 
     */
    public static NotificationMessageHolderType[] createNotificationMessage(
            CommonBaseEvent events[]) throws Exception {
        NotificationMessageHolderType[] notificationArray = 
            new NotificationMessageHolderType[events.length];
        for (int i = 0; i < events.length; i++) {

            //Creating an instance of NotificationMessageHolderType
            notificationArray[i] = new NotificationMessageHolderType();

            //Creating a Topic element with the name 'Topic'
            SOAPFactory soapFactory = new SOAPFactory();
            SOAPElement topicSE = (SOAPElement) 
                soapFactory.createElement("Topic");
            SOAPElement topicSEChild = (SOAPElement)
                topicSE.addChildElement("Topic");
            topicSEChild.setAttribute("dialect","none");
            notificationArray[i].setTopic(topicSE);

            //Setting the event to be the message of the notification
            SOAPElement messageSE = (SOAPElement) 
                soapFactory.createElement("Message");
            messageSE.addNamespaceDeclaration("ns2",
                "http://www.ibm.com/AC/commonbaseevent1_0_1");
            messageSE.addAttribute("http://www.w3.org/2001/XMLSchema-instance",
                "type",
                "ns2:CommonBaseEvent");
            String cbeStr = EventFormatter.toCanonicalXMLString(events[i]);
            SOAPElement cbeSE = (SOAPElement) 
                soapFactory.createElementFromXMLString(cbeStr);
            messageSE.addChildElement(cbeSE);
            notificationArray[i].setMessage(messageSE);

            //Setting information about the producer of the event in
            //the notification
            SOAPElement producerSE = (SOAPElement) 
                soapFactory.createElement("ProducerReference");
            SOAPElement producerSEChild = (SOAPElement) 
                soapFactory.createElement("Address",
                "ns1",
                "http://schemas.xmlsoap.org/ws/2003/03/addressing");
            producerSEChild.addTextNode("protocol://your.event.source.address");
            producerSE.addChildElement(producerSEChild);
            notificationArray[i].setProducerReference(producerSE);
        }
        return notificationArray;
    }

}
